//______________________________________________________________________
// P REISS	 Arrons	  arrondissement d' un flottant
// X = Arrons ( ToBe, PuDix, Ich )
//______________________________________________________________________
// entrees:
//   R*4 ToBe = nombre a arrondir
//   I*4 Ich  = choix de traitement:
//            = 0 => abs(ToBe) approche par defaut en ne
//                   gardant dans Arrons que le 1er chiffre
//                   significatif = partie entiere de
//                                  abs(ToBe) * PuDix
//            = 1 => abs(ToBe) approche par exces par X/PuDix,
//                   avec X = 1., 2. ou 5.
//                   ( X/PuDix = 10**(-10) si ToBe nul )
//            = 2 => abs(ToBe) approche par exces par 1./PuDix
//            = 3 => abs(ToBe) approche par exces par X/PuDix,
//                   avec X = 1., 1.25, 1.6, 2., 2.5, 3.2, 4.,
//                            5., 6.25 ou 8.
//                   ( X/PuDix = 10**(-10) si ToBe nul )
//______________________________________________________________________
// sorties:
//   R*4 PuDix  = puissance entiere de 10. telle que
//                1./PuDix <= abs(ToBe) < 10./PuDix
//   R*4 Arrons = partie sans exposant de l' approximation de ToBe
//______________________________________________________________________
// le cas Ich = 0 equivaut en gros a:
//   PuDix  = 10.**( -aint ( alog10 ( abs(ToBe) ) ) )
//   Arrons = aint ( abs(ToBe) * PuDix )
// en moins cher et en plus protege!

#include "math.h"
#include "Pro.h"

extern "C" double Arrons ( double *ToBe, double *PuDix, int *Ich )
  {
  double Monnaie[4] = { 1., 5., 2., 1. },
         Renard[11] = { 1., 8., 6.25, 5., 4., 3.2, 2.5, 2.,
                        1.6, 1.25, 1. };
  double Abs, Sortie, Pui, *Serie;
  int    Idx, Start, Cnt;

  Abs    = fabs ( *ToBe );		// 1 _ recherche de l' exposant
  *PuDix = 1.e10;
  Sortie = *Ich < 1   ?   *Ich   :   1;	// min ( *Ich, 1 )
  if ( Abs == 0. )   return Sortie;
  *PuDix = 1.;
  Pui    = 10.;
  if ( Abs >= 10. )   Pui = .1;
  while ( Abs < 1.   ||   Abs >= 10. )
    {
    Abs    *= Pui;
    *PuDix *= Pui;
    }
  Sortie = floor ( Abs );		// 2 _ recherche de la mantisse
  Start = 1;
  switch ( *Ich )
    {
    case 0:
      return Sortie;
    case 2:
      Start = 3;
    case 1:
      Cnt = 4;
      Serie = Monnaie;
      break;
    case 3:
      Cnt = 11;
      Serie = Renard;
    }
  for ( Idx = Start;   Idx < Cnt   &&   Abs <= Serie[Idx];   Idx++ )
    if ( Abs == Serie[Idx] )   return Sortie;
  Sortie = Serie[Idx - Start];
  if ( Idx == Start )   *PuDix *= .1;
  return Sortie;
  }
//______________________________________________________________________
